import unittest
from .chatactiontarget import ChatActionTarget

class ChatActionTargetTests(unittest.TestCase):
    def test_callService(self):
        """Test creation of a callService target."""
        at = ChatActionTarget(key='actiontarget-key', target='callService', parameters={'url': 'my-action-target-service'})

        at_descriptor = at.to_descriptor()
        expected = {'key': 'actiontarget-key', 'callService':{'url':'my-action-target-service'}}

        self.assertEqual(at_descriptor, expected)

    def test_openConfiguration(self):
        """Test creation of a openConfiguration target."""
        at = ChatActionTarget(key='actiontarget-key', target='openConfiguration', parameters={'key': 'config-key'})

        at_descriptor = at.to_descriptor()
        expected = {'key': 'actiontarget-key', 'openConfiguration':{'key':'config-key'}}

        self.assertEqual(at_descriptor, expected)

    def test_openDialog(self):
        """Test creation of a openDialog target."""
        at = ChatActionTarget(key='actiontarget-key', target='openDialog', parameters={'key': 'dialog-key'})

        at_descriptor = at.to_descriptor()
        expected = {'key': 'actiontarget-key', 'openDialog':{'key':'dialog-key'}}

        self.assertEqual(at_descriptor, expected)

    def test_openExternalPage(self):
        """Test creation of a openExternalPage target."""
        at = ChatActionTarget(key='actiontarget-key', target='openExternalPage', parameters={'key': 'external-page-key'})

        at_descriptor = at.to_descriptor()
        expected = {'key': 'actiontarget-key', 'openExternalPage':{'key':'external-page-key'}}

        self.assertEqual(at_descriptor, expected)

    def test_openSidebar(self):
        """Test creation of a openSidebar target."""
        at = ChatActionTarget(key='actiontarget-key', target='openSidebar', parameters={'key': 'sidebar-key'})

        at_descriptor = at.to_descriptor()
        expected = {'key': 'actiontarget-key', 'openSidebar':{'key':'sidebar-key'}}

        self.assertEqual(at_descriptor, expected)



if __name__ == '__main__':
    unittest.main()



from .module import Module

class ChatExternalPage(Module):
    type = 'chat:externalPage'

    def __init__(self, key, name, url, i18n=None):
        super().__init__(key)
        name_dict = {'value': name}
        if i18n is not None:
            name_dict['i18n'] = i18n
        self.add_property('name', name_dict)
        self.add_property('url', url)



from .module import Module

class ConditionClause(Module):
    clause_type = None

    def __init__(self, invert=None):
        if invert is not None:
            if not isinstance(invert, bool):
                raise ValueError('Unknown invert value: {}.'.format(invert))
            self.add_property('invert', invert)

class BuiltinConditionClause(ConditionClause):
    clause_type = 'built-in'
    valid_conditions = ['conversation_is_public', 'conversation_is_room']

    def __init__(self, condition, invert=None):
        super().__init__(invert=invert)
        if condition not in self.valid_conditions:
            raise ValueError('Unknown builtin condition: {}'.format(condition))

        self.add_property('condition', condition)

class CompositeConditionClause(ConditionClause):
    clause_type = 'composite'
    valid_types = ['or', 'OR', 'and', 'AND']

    def __init__(self, type, conditions, invert=None):
        super().__init__(invert=invert)

        # Validate the type parameter
        if type not in self.valid_types:
            raise ValueError('Unknown composite condition type: {}'.format(type))
        self.add_property('type', type.lower())

        # Validate the conditions parameter
        if isinstance(conditions, list) and len(conditions)<1:
            raise ValueError('Composite condition clause must have at least condition.')
        elif isinstance(conditions, ConditionClause):
            conditions = [ conditions ]
        elif isinstance(conditions, list):
            for c in conditions:
                if not isinstance(c, ConditionClause):
                    raise ValueError('Invalid condition clause: {}'.format(c))
        else:
            raise ValueError('Invalid condition type: {}.'.format(type(conditions)))
        self.add_property('conditions', conditions)



from .module import Module

class ChatBot(Module):
    type = 'chat:bot'

    def __init__(self, key, mention_path=None, direct_message_path=None):
        super().__init__(key)
        if mention_path is not None:
            self.add_property('mention', {'url': mention_path})
        if direct_message_path is not None:
            self.add_property('directMessage', {'url': direct_message_path})



from .module import Module

class ChatActionTarget(Module):
    type = 'chat:actionTarget'

    # Target types allowed for "static" actions
    valid_targets = [ 'callService', 'openConfiguration', 'openDialog', 'openExternalPage', 'openSidebar' ]

    def __init__(self, key, target, parameters={}):
        if target not in self.valid_targets:
            raise ValueError('Invalid Action Target: '+target)

        super().__init__(key)

        target_parameters = []
        if target == 'callService':
            target_parameters = ["url"]
        elif target == 'openConfiguration':
            target_parameters = ["key"]
        elif target == 'openDialog':
            target_parameters = ["key"]
        elif target == 'openExternalPage':
            target_parameters = ["key"]
        elif target == 'openSidebar':
            target_parameters = ["key"]

        params = {}

        for tp in target_parameters:
            params[tp] = parameters[tp]

        self.add_property(target, params)


from .module import Module

class ChatSidebar(Module):
    type = 'chat:sidebar'

    def __init__(self, key, name, url, icon_1x=None, icon_2x=None, i18n=None, weight=None, conditions=None):
        super().__init__(key)
        name_dict = { 'value': name }
        if i18n is not None:
            name_dict['i18n'] = i18n
        self.add_property('name', name_dict)

        self.add_property('url', url)

        if icon_1x is not None:
            icon_dict = { 'url': icon_1x, 'url@2x': icon_2x }
            self.add_property('icon', icon_dict)
        
        if weight is not None:
            self.add_property('weight', weight)
        
        if conditions is not None:
            self.add_property('conditions', conditions)



from functools import wraps
import uuid

import stride
from flask import Blueprint, request, jsonify

from .module import Module
from .chatbot import ChatBot
from .chatbotmessage import ChatBotMessage
from .chatwebhook import ChatWebhook
from .chatdialog import ChatDialog
from .chatmessageaction import ChatMessageAction
from .chatinputaction import ChatInputAction
from .chatactiontarget import ChatActionTarget
from .chatexternalpage import ChatExternalPage
from .chatglance import ChatGlance
from .chatsidebar import ChatSidebar
from .chatconfiguration import ChatConfiguration

class Stride(stride.Stride):
    def __init__(self, key=None, app=None, url_prefix=''):
        self.stride_key = key
        self.app = app
        super().__init__(cloud_id=None, client_id=None, client_secret=None)
        self.url_prefix = url_prefix
        self.lifecycle = {}
        self.modules = {}
        blueprint_name = 'stride_{}'.format(self.stride_key)
        static_file_path = '{}/static'.format(url_prefix)
        self._blueprint = Blueprint(blueprint_name, __name__, static_url_path=static_file_path)

    def init_app(self, app):
        # In case we didn't pass the Flask app in the constructor
        if app is not None:
            self.app = app

        if self.stride_key is None:
            self.stride_key = app.config['STRIDE_DESCRIPTOR_KEY']
        self.cloud_id = app.config['STRIDE_CLOUD_ID']
        self.client_id = app.config['STRIDE_CLIENT_ID']
        self.client_secret = app.config['STRIDE_CLIENT_SECRET']

        if 'STRIDE_DESCRIPTOR_URL' in app.config:
            self.descriptor_path = app.config['STRIDE_DESCRIPTOR_URL']
        else:
            self.descriptor_path = '/app-descriptor.json'

        self._blueprint.add_url_rule(self.descriptor_path, 'app_descriptor', self.app_descriptor)

        if 'installed' not in self.lifecycle:
            @self.installed()
            def installed_default():
                return ('',204)

        if 'uninstalled'  not in self.lifecycle:
            @self.uninstalled()
            def uninstalled_default():
                return ('', 204)

        # This must be done last
        self.app.register_blueprint(self._blueprint, url_prefix=self.url_prefix)

    def _get_module(self, key):
        for mod_type in self.modules:
            for m in self.modules[mod_type]:
                if m.properties['key'] == key:
                    return m
        return None

    def _add_module(self, module):
        """Register a module to the app."""
        # Make sure we are dealing with a Module object
        if not isinstance(module, Module):
            pass

        module_type = module.type
        mod_key = module.properties['key']

        # Ensure that all modules added have a unique key.
        if self._get_module(mod_key):
            raise ValueError('Module with key "{}" already exists.'.format(mod_key))


        # If this module is the first of its type, we need to add the key to the dict.
        if module_type not in self.modules:
            self.modules[module_type] = []

        self.modules[module_type].append(module)

    def _del_module(self, key):
        mod = self._get_module(key)

        # If there is no module with that key, do nothing.
        if mod is None:
            return

        mod_type = mod.type
        self.modules[mod_type].remove(mod)

        # If there are no more modules of this type, delete the type.
        if len(self.modules[mod_type]) == 0:
            del(self.modules[mod_type])

    def chat_bot(self, key=None, mention_path=None, direct_message_path=None):
        """Decorator for declaring a function to run against a chat:bot module."""
        def module_decorator(func):
            module = ChatBot(key, mention_path, direct_message_path)
            self._add_module(module)

            @wraps(func)
            def func_wrapper(*args, **kwargs):
                return func(*args, **kwargs)
            if direct_message_path is not None:
                func_wrapper = self._blueprint.route(
                    direct_message_path, methods=['POST'])(func_wrapper)
            if mention_path is not None:
                func_wrapper = self._blueprint.route(
                    mention_path, methods=['POST'])(func_wrapper)
            return func_wrapper
        return module_decorator

    def chat_bot_messages(self, key=None, pattern=None, path=None):
        """Decorator for declaring a function to run against a chat:bot:messages module."""
        def module_decorator(func):
            module = ChatBotMessage(key, pattern, path)
            self._add_module(module)

            @self._blueprint.route(path, methods=['POST'])
            @wraps(func)
            def func_wrapper(*args, **kwargs):
                return func(*args, **kwargs)
            return func_wrapper
        return module_decorator

    def chat_webhook(self, key, url, event='conversation:updates'):
        """Add a webhook."""
        module = ChatWebhook(key, event, url)
        self._add_module(module)

    def chat_dialog(self, key, title, path, size=None, filter=None, hint=None, primary_action=None, secondary_actions=None):
        """Decorator for declaring a function to run against a chat:dialog module."""
        def module_decorator(func):
            module = ChatDialog(key=key, title=title, url=path, size=size, filter=filter, hint=hint, primary_action=primary_action, secondary_actions=secondary_actions)
            self._add_module(module)

            @self._blueprint.route(path, methods=['GET'])
            @wraps(func)
            def func_wrapper(*args, **kwargs):
                return func(*args, **kwargs)
            return func_wrapper
        return module_decorator

    def chat_message_action(self, key, name, target, weight=None, conditions=None):
        """Add a message action."""
        module = ChatMessageAction(key, name, target, weight, conditions)
        self._add_module(module)

    def chat_input_action(self, key, name, target, weight=None, conditions=None):
        """Add a input action."""
        module = ChatInputAction(key, name, target, weight, conditions)
        self._add_module(module)

    def chat_external_page(self, key, name, url, i18n=None):
        """Add an external page."""
        module = ChatExternalPage(key=key, name=name, url=url, i18n=i18n)
        self._add_module(module)

    def chat_glance(self, key, name, icon_1x, icon_2x, i18n=None, target=None, weight=None, conditions=None):
        """Basic chat:glance object without the queryUrl field."""
        module = ChatGlance(key=key, name=name, icon_1x=icon_1x, icon_2x=icon_2x, i18n=i18n, target=target, weight=weight, conditions=conditions)
        self._add_module(module)

    def chat_glance_query(self, key, path):
        """Decorator for declaring the queryUrl endpoint of the chat:glance module."""
        def module_decorator(func):
            glance = self._get_module(key)

            if glance is None or glance.type != 'chat:glance':
                raise Error('Glance "{}" is not defined.'.format(key))

            glance.properties['queryUrl'] = path

            @self._blueprint.route(path, methods=['GET'])
            @wraps(func)
            def func_wrapper(*args, **kwargs):
                return func(*args, **kwargs)
            return func_wrapper
        return module_decorator

    def chat_sidebar(self, key, name, url, icon_1x=None, icon_2x=None, i18n=None, weight=None, conditions=None):
        """Decorator for declaring a chat:sidebar module."""
        def module_decorator(func):
            module = ChatSidebar(key=key, name=name, url=url, icon_1x=icon_1x, icon_2x=icon_2x, i18n=i18n, weight=weight, conditions=conditions)
            self._add_module(module)

            @self._blueprint.route(url, methods=['GET'])
            @wraps(func)
            def func_wrapper(*args, **kwargs):
                return func(*args, **kwargs)
            return func_wrapper
        return module_decorator


    def chat_configuration_url(self, key, url, external_page=False):
        """Create a chat:configuration module and declare the (external)page.url endpoint."""
        def module_decorator(func):
            if external_page:
                module = ChatConfiguration(key=key, externalpage_url=url)
            else:
                module = ChatConfiguration(key=key, page_url=url)
            self._add_module(module)

            @self._blueprint.route(url, methods=['GET'])
            @wraps(func)
            def func_wrapper(*args, **kwargs):
                return func(*args, **kwargs)
            return func_wrapper
        return module_decorator

    def chat_configuration_target(self, key, target):
        """Add a chat:configuration module that points to a target."""
        module = ChatConfiguration(key=key, target=target)
        self._add_module(module)

    def chat_action_target(self, key, target, parameters={}):
        """Add a chat:actionTarget module"""
        module = ChatActionTarget(key=key, target=target, parameters=parameters)
        self._add_module(module)

    def installed(self, path='/installed'):
        """Decorator for declaring a function to run as part of the install lifecycle."""
        def installed_decorator(func):
            self.lifecycle['installed'] = path
            @self._blueprint.route(path, methods=['POST'])
            @wraps(func)
            def func_wrapper(*args, **kwargs):
                return func(*args, **kwargs)
            return func_wrapper
        return installed_decorator

    def uninstalled(self, path='/uninstalled'):
        """Decorator for declaring a function to run as part of the uninstall lifecycle."""
        def uninstalled_decorator(func):
            self.lifecycle['uninstalled'] = path
            @self._blueprint.route(path, methods=['POST'])
            @wraps(func)
            def func_wrapper(*args, **kwargs):
                return func(*args, **kwargs)
            return func_wrapper
        return uninstalled_decorator

    def app_descriptor(self):
        """Generate the app descriptor JSON text that Atlassian Connect is expecting."""
        descriptor = {}
        base_url = 'https://{}{}'.format(request.host, self.url_prefix)
        descriptor['baseUrl'] = base_url
        descriptor['key'] = self.stride_key
        descriptor['lifecycle'] = {}
        for lifecycle_prop in ['installed', 'uninstalled']:
            if lifecycle_prop in self.lifecycle:
                descriptor['lifecycle'][lifecycle_prop] = self.lifecycle[lifecycle_prop]
            else:
                descriptor['lifecycle'][lifecycle_prop] = '/{}'.format(lifecycle_prop)

        descriptor['modules'] = {}
        for module_type in self.modules:
          descriptor['modules'][module_type] = [ m.to_descriptor() for m in self.modules[module_type] if m is not None ]

        return jsonify(descriptor)



from .module import Module

class ChatDialog(Module):
    type = 'chat:dialog'

    def __init__(self, key, title, url, size=None, filter=None, hint=None, primary_action=None, secondary_actions=None):
        super().__init__(key)
        self.add_property('title', {'value': title})
        self.add_property('url', url)
        if size is not None:
            self.size = size

        if hint is not None:
            self.hint = hint

        if filter is not None:
            self.filter = filter

        if primary_action is not None:
            self.primary_action = primary_action

        if secondary_actions is not None:
            self.secondary_actions = secondary_actions

    def add_option(self, name, value):
        if 'options' not in self.properties:
            self.add_property('options', {})

        self.properties['options'][name] = value

    def del_option(self, name):
        if 'options' in self.properties:
            if name in self.properties['options']:
                del(self.properties['options'][name])

        if self.properties['options'] == {}:
            self.del_property('options')

    @property
    def size(self):
        try:
            return self.properties['options']['size']
        except:
            return None

    @size.setter
    def size(self, value):
        """Specify the size of the dialog box.
           Accepts either a string or a dict.
           If `value` is a string, it must a modal dialog size supported by the Atlassian Design Guidelines
           If `value` is a dict, it must have exactly two fields: `width` and `height`. Each field value is a string representing a number followed by 'px' or '%'."""
        if isinstance(value, str):
            valid_sizes = ['small', 'medium', 'large', 'xlarge']
            if value not in valid_sizes:
                raise ValueError('{} is not a valid size.'.format(value))
        elif isinstance(value, dict):
            if (len(value) != 2) or ('width' not in value) or ('height' not in value):
                raise ValueError('Custom dialog size is not valid.')

            if not value['width'].endswith(('px', '%')):
                raise ValueError('Width must be in pixels (px) or as a percentage (%).')

            if not value['height'].endswith(('px', '%')):
                raise ValueError('Height must be in pixels (px) or as a percentage (%).')
        else:
            raise ValueError('Invalid size data.')

        self.add_option('size', value)

    @property
    def hint(self):
        try:
            return self.properties['options']['hint']
        except:
            return None

    @hint.setter
    def hint(self, value):
        """Set the hint for the dialog."""
        if isinstance(value, str):
            self.add_option('hint', {'value': value})
        elif isinstance(value, dict):
            self.add_option('hint', {'value': value['value'], 'i18n': value['i18n']})
        else:
            raise ValueError('Invalid hint value.')

    @property
    def filter(self):
        try:
            return self.properties['options']['filter']
        except:
            return None

    @filter.setter
    def filter(self, value):
        if not isinstance(value, dict):
            raise ValueError('Invalid filter data type. Expecting a dict.')

        placeholder = {}
        placeholder['value'] = value['placeholder']['value']
        if 'i18n' in value['placeholder']:
            placeholder['i18n'] = value['placeholder']['i18n']

        self.add_option('filter', {'placeholder': placeholder})


    @property
    def primary_action(self):
        try:
            return self.properties['options']['primaryAction']
        except:
            return None

    @primary_action.setter
    def primary_action(self, value):
        """Set the primary action"""
        default_key = 'primary-action-selected'

        if not isinstance(value, ChatDialogAction):
            raise ValueError('Invalid data type for Primary Action. Expecting a ChatDialogAction. Received {}'.format(type(value)))

        if getattr(value, 'key', None) is None:
            value.key = default_key

        self.add_option('primaryAction', value)

    @property
    def secondary_actions(self):
        try:
            return self.properties['options']['secondaryActions']
        except:
            return None

    @secondary_actions.setter
    def secondary_actions(self, actions):
        if isinstance(actions, ChatDialogAction):
            actions = [actions]

        if not isinstance(actions, list):
            raise ValueError('Expecting a list. Received {}.'.format(type(actions)))

        for act in actions:
            if not isinstance(act, ChatDialogAction):
                raise ValueError('Expected a list of ChatDialogAction. Found list with element of type {}.'.format(type(act)))

        self.add_option('secondaryActions', actions)

    def add_secondary_action(self, action):
        """Add an action to the list of secondary actions."""
        if ('options' not in self.properties) or ('secondaryActions' not in self.properties['options']):
            self.add_option('secondaryActions', [])
        self.properties['options']['secondaryActions'].append(action)

    def del_secondary_action(self, name):
        """Remove a secondary action based on a name."""
        new_actions_list = [ a for a in self.secondary_actions if a.properties['name']['value'] != name ]

        if len(new_actions_list) == 0:
            self.del_option('secondaryActions')
        else:
            self.add_option('secondaryActions', new_actions_list)

class ChatDialogAction(Module):
    def __init__(self, name, key=None, enabled=True, i18n=None):
        super().__init__(key)
        self.add_property('name', {'value': name})
        self.add_property('enabled', enabled)

        if i18n is not None:
            self.properties['options']['name']['i18n'] = i18n



import unittest
from .chatsidebar import ChatSidebar

class ChatSidebarTests(unittest.TestCase):
    def test_init(self):
        """Test the __init__() method."""
        sb = ChatSidebar(key='sidebar-key', name='My Sidebar', icon_1x='https://my.icons/1x', icon_2x='https://my.icons/2x', url='/mysidebar')

        sb_descriptor = sb.to_descriptor()
        expected = {'key': 'sidebar-key', 'name':{'value':'My Sidebar'}, 'icon':{'url':'https://my.icons/1x', 'url@2x':'https://my.icons/2x'}, 'url':'/mysidebar'}

        self.assertEqual(sb_descriptor, expected)

if __name__ == '__main__':
    unittest.main()



import unittest
from .chatglance import ChatGlance

class ChatGlanceTests(unittest.TestCase):
    def test_init(self):
        """Test the __init__() method."""
        gl = ChatGlance(key='glance-key', name='My Glance', icon_1x='https://my.icons/1x', icon_2x='https://my.icons/2x')

        gl_descriptor = gl.to_descriptor()
        expected = {'key': 'glance-key', 'name':{'value':'My Glance'}, 'icon':{'url':'https://my.icons/1x', 'url@2x':'https://my.icons/2x'}} 

        self.assertEqual(gl_descriptor, expected)

if __name__ == '__main__':
    unittest.main()
